@@ -1,7 +1,7 @@
-/* $Id: control.cc 1558 2012-11-29 18:59:26Z dreibh $
+/* $Id: control.cc 1880 2014-07-10 12:35:32Z dreibh $
  *
  * Network Performance Meter
- * Copyright (C) 2009-2012 by Thomas Dreibholz
+ * Copyright (C) 2009-2014 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Contact: dreibh@iem.uni-due.de
+ * Contact: sebastian.wallat@uni-due.de, dreibh@iem.uni-due.de
  */
 
 #include "control.h"
@@ -154,13 +154,11 @@ bool performNetPerfMeterAddFlow(MessageReader* messageReader,
    NetPerfMeterAddFlowMessage* addFlowMsg = (NetPerfMeterAddFlowMessage*)&addFlowMsgBuffer;
    addFlowMsg->Header.Type   = NETPERFMETER_ADD_FLOW;
    addFlowMsg->Header.Flags  = 0x00;
-   addFlowMsg->CCID          = flow->getTrafficSpec().CCID;
-   addFlowMsg->CMT           = flow->getTrafficSpec().CMT;
-   if(flow->getTrafficSpec().CMT > 0) {
-      addFlowMsg->Header.Flags |= NPMAF_USE_CMT;   /* Backward compatibility */
+   if(flow->getTrafficSpec().Debug == true) {
+      addFlowMsg->Header.Flags |= NPMAFF_DEBUG;
    }
-   if(flow->getTrafficSpec().CCID > 0) {
-      addFlowMsg->Header.Flags |= NPMAF_USE_RP;    /* Backward compatibility */
+   if(flow->getTrafficSpec().NoDelay == true) {
+      addFlowMsg->Header.Flags |= NPMAFF_NODELAY;
    }
 
    addFlowMsg->Header.Length = htons(addFlowMsgSize);
@@ -168,7 +166,7 @@ bool performNetPerfMeterAddFlow(MessageReader* messageReader,
    addFlowMsg->FlowID        = htonl(flow->getFlowID());
    addFlowMsg->StreamID      = htons(flow->getStreamID());
    addFlowMsg->Protocol      = flow->getTrafficSpec().Protocol;
-   addFlowMsg->Flags         = 0x00;
+   addFlowMsg->pad           = 0x00;
    for(size_t i = 0;i < NETPERFMETER_RNG_INPUT_PARAMETERS;i++) {
       addFlowMsg->FrameRate[i] = doubleToNetwork(flow->getTrafficSpec().InboundFrameRate[i]);
       addFlowMsg->FrameSize[i] = doubleToNetwork(flow->getTrafficSpec().InboundFrameSize[i]);
@@ -183,6 +181,8 @@ bool performNetPerfMeterAddFlow(MessageReader* messageReader,
    addFlowMsg->RetransmissionTrials =
       htonl((uint32_t)flow->getTrafficSpec().RetransmissionTrials |
             (uint32_t)(flow->getTrafficSpec().RetransmissionTrialsInMS ? NPMAF_RTX_TRIALS_IN_MILLISECONDS : 0));
+   addFlowMsg->CCID          = flow->getTrafficSpec().CCID;
+   addFlowMsg->CMT           = flow->getTrafficSpec().CMT;
 
    addFlowMsg->OnOffEvents   = htons(flow->getTrafficSpec().OnOffEvents.size());
    size_t i = 0;
@@ -194,6 +194,11 @@ bool performNetPerfMeterAddFlow(MessageReader* messageReader,
    strncpy((char*)&addFlowMsg->Description, flow->getTrafficSpec().Description.c_str(),
            std::min(sizeof(addFlowMsg->Description), flow->getTrafficSpec().Description.size()));
 
+   addFlowMsg->NDiffPorts = htons(flow->getTrafficSpec().NDiffPorts);
+   memset((char*)&addFlowMsg->PathMgr, 0, sizeof(addFlowMsg->PathMgr));
+   strncpy((char*)&addFlowMsg->PathMgr, flow->getTrafficSpec().PathMgr.c_str(),
+           std::min(sizeof(addFlowMsg->PathMgr), flow->getTrafficSpec().PathMgr.size()));
+
    sctp_sndrcvinfo sinfo;
    memset(&sinfo, 0, sizeof(sinfo));
    sinfo.sinfo_ppid = htonl(PPID_NETPERFMETER_CONTROL);
@@ -202,7 +207,7 @@ bool performNetPerfMeterAddFlow(MessageReader* messageReader,
       std::cout << "<R1> "; std::cout.flush();
    }
    if(sctp_send(controlSocket, addFlowMsg, addFlowMsgSize, &sinfo, 0) <= 0) {
-      perror("ERROR");
+      perror("sctp_send error");
       return(false);
    }
 
@@ -213,7 +218,7 @@ bool performNetPerfMeterAddFlow(MessageReader* messageReader,
    if(awaitNetPerfMeterAcknowledge(messageReader, controlSocket,
                                    flow->getMeasurementID(),
                                    flow->getFlowID(), flow->getStreamID()) == false) {
-      perror("ERROR");
+      perror("sctp_rev error");
       return(false);
    }
 
@@ -346,6 +351,10 @@ bool performNetPerfMeterStart(MessageReader*         messageReader,
          fprintf(configFile, "FLOW%u_INBOUND_FRAME_SIZE_RNG=%u\n",           flow->getFlowID(), flow->getTrafficSpec().InboundFrameSizeRng);
          fprintf(configFile, "FLOW%u_RELIABLE=%f\n",                         flow->getFlowID(), flow->getTrafficSpec().ReliableMode);
          fprintf(configFile, "FLOW%u_ORDERED=%f\n",                          flow->getFlowID(), flow->getTrafficSpec().OrderedMode);
+         fprintf(configFile, "FLOW%u_NODELAY=\"%s\"\n",                      flow->getFlowID(), (flow->getTrafficSpec().NoDelay == true) ? "on" : "off");
+         fprintf(configFile, "FLOW%u_DEBUG=\"%s\"\n",                        flow->getFlowID(), (flow->getTrafficSpec().Debug == true) ? "on" : "off");
+         fprintf(configFile, "FLOW%u_PATHMGR=\"%s\"\n",                      flow->getFlowID(), flow->getTrafficSpec().PathMgr.c_str());
+         fprintf(configFile, "FLOW%u_NDIFFPORTS=%u\n",                       flow->getFlowID(), flow->getTrafficSpec().NDiffPorts);
          fprintf(configFile, "FLOW%u_VECTOR_ACTIVE_NODE=\"%s\"\n",           flow->getFlowID(), flow->getVectorFile().getName().c_str());
          fprintf(configFile, "FLOW%u_VECTOR_PASSIVE_NODE=\"%s\"\n\n",        flow->getFlowID(),
                               Flow::getNodeOutputName(vectorNamePattern, "passive",
@@ -614,9 +623,8 @@ bool awaitNetPerfMeterAcknowledge(MessageReader* messageReader,
 
 
 // ###### Upload file #######################################################
-static bool uploadOutputFile(const int          controlSocket,
-                             const sctp_assoc_t assocID,
-                             const OutputFile&  outputFile)
+static bool uploadOutputFile(const int         controlSocket,
+                             const OutputFile& outputFile)
 {
    char                 messageBuffer[sizeof(NetPerfMeterResults) +
                                       NETPERFMETER_RESULTS_MAX_DATA_LENGTH];
@@ -625,8 +633,7 @@ static bool uploadOutputFile(const int          controlSocket,
    // ====== Initialize header ==============================================
    sctp_sndrcvinfo sinfo;
    memset(&sinfo, 0, sizeof(sinfo));
-   sinfo.sinfo_assoc_id    = assocID;
-   sinfo.sinfo_ppid        = htonl(PPID_NETPERFMETER_CONTROL);
+   sinfo.sinfo_ppid = htonl(PPID_NETPERFMETER_CONTROL);
    resultsMsg->Header.Type = NETPERFMETER_RESULTS;
 
    if(gOutputVerbosity >= NPFOV_CONNECTIONS) {
@@ -669,7 +676,7 @@ static bool uploadOutputFile(const int          controlSocket,
       std::cerr << std::endl
                 << "WARNING: Unable to upload results (errno=" << errno << "): "
                 << strerror(errno) << std::endl;
-      sendAbort(controlSocket, assocID);
+      sendAbort(controlSocket);
    }
    if(gOutputVerbosity >= NPFOV_CONNECTIONS) {
       std::cout << " " << ((success == true) ? "okay": "FAILED") << std::endl;
@@ -679,20 +686,19 @@ static bool uploadOutputFile(const int          controlSocket,
 
 
 // ###### Upload per-flow statistics file ###################################
-static bool uploadResults(const int          controlSocket,
-                          const sctp_assoc_t assocID,
-                          Flow*              flow)
+static bool uploadResults(const int controlSocket,
+                          Flow*     flow)
 {
    bool success = flow->getVectorFile().finish(false);
    if(success) {
       success = sendNetPerfMeterAcknowledge(
-                   controlSocket, assocID,
+                   controlSocket,
                    flow->getMeasurementID(), flow->getFlowID(), flow->getStreamID(),
                    (success == true) ? NETPERFMETER_STATUS_OKAY :
                                        NETPERFMETER_STATUS_ERROR);
       if(success) {
          if(flow->getVectorFile().exists()) {
-            success = uploadOutputFile(controlSocket, assocID, flow->getVectorFile());
+            success = uploadOutputFile(controlSocket, flow->getVectorFile());
          }
       }
    }
@@ -705,7 +711,6 @@ static bool uploadResults(const int          controlSocket,
 static bool handleNetPerfMeterAddFlow(MessageReader*                    messageReader,
                                       const int                         controlSocket,
                                       const NetPerfMeterAddFlowMessage* addFlowMsg,
-                                      const sctp_assoc_t                assocID,
                                       const size_t                      received)
 {
    if(received < sizeof(NetPerfMeterAddFlowMessage)) {
@@ -728,15 +733,15 @@ static bool handleNetPerfMeterAddFlow(MessageReader*                    messageR
    if(FlowManager::getFlowManager()->findFlow(measurementID, flowID, streamID)) {
       std::cerr << "ERROR: NETPERFMETER_ADD_FLOW tried to add already-existing flow!"
                 << std::endl;
-      return(sendNetPerfMeterAcknowledge(controlSocket, assocID,
+      return(sendNetPerfMeterAcknowledge(controlSocket,
                                          measurementID, flowID, streamID,
                                          NETPERFMETER_STATUS_ERROR));
    }
    else {
       // ====== Create new flow =============================================
       FlowTrafficSpec trafficSpec;
-      trafficSpec.Protocol             = addFlowMsg->Protocol;
-      trafficSpec.Description          = std::string(description);
+      trafficSpec.Protocol    = addFlowMsg->Protocol;
+      trafficSpec.Description = std::string(description);
       for(size_t i = 0;i < NETPERFMETER_RNG_INPUT_PARAMETERS;i++) {
          trafficSpec.OutboundFrameRate[i] = networkToDouble(addFlowMsg->FrameRate[i]);
          trafficSpec.OutboundFrameSize[i] = networkToDouble(addFlowMsg->FrameSize[i]);
@@ -754,25 +759,26 @@ static bool handleNetPerfMeterAddFlow(MessageReader*                    messageR
       if( (trafficSpec.RetransmissionTrialsInMS) && (trafficSpec.RetransmissionTrials == 0x7fffffff) ) {
          trafficSpec.RetransmissionTrials = ~0;
       }
+      const uint32_t* events = (const uint32_t*)&addFlowMsg->OnOffEvent;
       for(size_t i = 0;i < startStopEvents;i++) {
-         trafficSpec.OnOffEvents.insert(ntohl(addFlowMsg->OnOffEvent[i]));
+         trafficSpec.OnOffEvents.insert(ntohl(events[i]));
       }
       trafficSpec.CMT  = addFlowMsg->CMT;
       trafficSpec.CCID = addFlowMsg->CCID;
 
-      if(trafficSpec.CMT == 0x00) {   /* Backward compatibility */
-         if(addFlowMsg->Header.Flags & (NPMAF_USE_CMT|NPMAF_USE_RP)) {
-            trafficSpec.CMT = NPAF_CMTRPv1;
-         }
-         else if(addFlowMsg->Header.Flags & NPMAF_USE_CMT) {
-            trafficSpec.CMT = NPAF_CMT;
-         }
-      }
+      trafficSpec.NoDelay = (addFlowMsg->Header.Flags & NPMAFF_NODELAY);
+      trafficSpec.Debug   = (addFlowMsg->Header.Flags & NPMAFF_DEBUG);
+
+      char pathMgr[sizeof(addFlowMsg->PathMgr) + 1];
+      memcpy((char*)&pathMgr, (const char*)&addFlowMsg->PathMgr, sizeof(addFlowMsg->PathMgr));
+      pathMgr[sizeof(addFlowMsg->PathMgr)] = 0x00;
+      trafficSpec.PathMgr    = std::string(pathMgr);
+      trafficSpec.NDiffPorts = ntohs(addFlowMsg->NDiffPorts);
 
       Flow* flow = new Flow(ntoh64(addFlowMsg->MeasurementID), ntohl(addFlowMsg->FlowID),
                             ntohs(addFlowMsg->StreamID), trafficSpec,
-                            controlSocket, assocID);
-      return(sendNetPerfMeterAcknowledge(controlSocket, assocID,
+                            controlSocket);
+      return(sendNetPerfMeterAcknowledge(controlSocket,
                                          measurementID, flowID, streamID,
                                          (flow != NULL) ? NETPERFMETER_STATUS_OKAY :
                                                           NETPERFMETER_STATUS_ERROR));
@@ -784,7 +790,6 @@ static bool handleNetPerfMeterAddFlow(MessageReader*                    messageR
 static bool handleNetPerfMeterRemoveFlow(MessageReader*                       messageReader,
                                          const int                            controlSocket,
                                          const NetPerfMeterRemoveFlowMessage* removeFlowMsg,
-                                         const sctp_assoc_t                   assocID,
                                          const size_t                         received)
 {
    if(received < sizeof(NetPerfMeterRemoveFlowMessage)) {
@@ -799,7 +804,7 @@ static bool handleNetPerfMeterRemoveFlow(MessageReader*                       me
    if(flow == NULL) {
       std::cerr << "ERROR: NETPERFMETER_ADD_REMOVE tried to remove not-existing flow!"
                 << std::endl;
-      return(sendNetPerfMeterAcknowledge(controlSocket, assocID,
+      return(sendNetPerfMeterAcknowledge(controlSocket,
                                          measurementID, flowID, streamID,
                                          NETPERFMETER_STATUS_ERROR));
    }
@@ -809,7 +814,7 @@ static bool handleNetPerfMeterRemoveFlow(MessageReader*                       me
       // ------ Upload statistics file --------------------------------
       flow->getVectorFile().finish(false);
       if(flow->getVectorFile().exists()) {
-         uploadResults(controlSocket, assocID, flow);
+         uploadResults(controlSocket, flow);
       }
 
       delete flow;
@@ -822,7 +827,6 @@ static bool handleNetPerfMeterRemoveFlow(MessageReader*                       me
 static bool handleNetPerfMeterStart(MessageReader*                  messageReader,
                                     const int                       controlSocket,
                                     const NetPerfMeterStartMessage* startMsg,
-                                    const sctp_assoc_t              assocID,
                                     const size_t                    received)
 {
    if(received < sizeof(NetPerfMeterStartMessage)) {
@@ -858,7 +862,7 @@ static bool handleNetPerfMeterStart(MessageReader*                  messageReade
       NULL, scalarFileFormat,
       (gOutputVerbosity >= NPFOV_FLOWS));
 
-   return(sendNetPerfMeterAcknowledge(controlSocket, assocID,
+   return(sendNetPerfMeterAcknowledge(controlSocket,
                                       measurementID, 0, 0,
                                       (success == true) ? NETPERFMETER_STATUS_OKAY :
                                                           NETPERFMETER_STATUS_ERROR));
@@ -869,7 +873,6 @@ static bool handleNetPerfMeterStart(MessageReader*                  messageReade
 static bool handleNetPerfMeterStop(MessageReader*                 messageReader,
                                    const int                      controlSocket,
                                    const NetPerfMeterStopMessage* stopMsg,
-                                   const sctp_assoc_t             assocID,
                                    const size_t                   received)
 {
    if(received < sizeof(NetPerfMeterStopMessage)) {
@@ -910,7 +913,7 @@ static bool handleNetPerfMeterStop(MessageReader*                 messageReader,
    FlowManager::getFlowManager()->unlock();
 
    // ====== Acknowledge result =============================================
-   sendNetPerfMeterAcknowledge(controlSocket, assocID,
+   sendNetPerfMeterAcknowledge(controlSocket,
                                measurementID, 0, 0,
                                (success == true) ? NETPERFMETER_STATUS_OKAY :
                                                    NETPERFMETER_STATUS_ERROR);
@@ -919,10 +922,10 @@ static bool handleNetPerfMeterStop(MessageReader*                 messageReader,
    if(measurement) {
       if(success) {
          if(measurement->getVectorFile().exists()) {
-            uploadOutputFile(controlSocket, assocID, measurement->getVectorFile());
+            uploadOutputFile(controlSocket, measurement->getVectorFile());
          }
          if(measurement->getScalarFile().exists()) {
-            uploadOutputFile(controlSocket, assocID, measurement->getScalarFile());
+            uploadOutputFile(controlSocket, measurement->getScalarFile());
          }
       }
       delete measurement;
@@ -932,17 +935,16 @@ static bool handleNetPerfMeterStop(MessageReader*                 messageReader,
 
 
 // ###### Delete all flows owned by a given remote node #####################
-static void handleControlAssocShutdown(const sctp_assoc_t assocID)
+static void handleControlAssocShutdown(int controlSocket)
 {
    FlowManager::getFlowManager()->lock();
 
    std::vector<Flow*>::iterator iterator = FlowManager::getFlowManager()->getFlowSet().begin();
    while(iterator != FlowManager::getFlowManager()->getFlowSet().end()) {
       Flow* flow = *iterator;
-      if(flow->getRemoteControlAssocID() == assocID) {
+      if(flow->getRemoteControlSocketDescriptor() == controlSocket) {
          Measurement* measurement = flow->getMeasurement();
          if(measurement) {
-            printf("delete2=%p\n", measurement);
             delete measurement;
          }
          delete flow;
@@ -975,8 +977,10 @@ bool handleNetPerfMeterControlMessage(MessageReader* messageReader,
       return(true);   // Partial read -> wait for next fragment.
    }
    else if(received < (ssize_t)sizeof(NetPerfMeterHeader)) {
-      std::cerr << "ERROR: Control connection is broken!" << std::endl;
-      sendAbort(controlSocket, sinfo.sinfo_assoc_id);
+      if(received < 0) { 
+         std::cerr << "ERROR: Control connection is broken!" << std::endl;
+         sendAbort(controlSocket, -1);
+      }
       return(false);
    }
 
@@ -986,7 +990,7 @@ bool handleNetPerfMeterControlMessage(MessageReader* messageReader,
       if( (notification->sn_header.sn_type == SCTP_ASSOC_CHANGE) &&
           ((notification->sn_assoc_change.sac_state == SCTP_COMM_LOST) ||
            (notification->sn_assoc_change.sac_state == SCTP_SHUTDOWN_COMP)) ) {
-        handleControlAssocShutdown(notification->sn_assoc_change.sac_assoc_id);
+        handleControlAssocShutdown(controlSocket);
       }
    }
 
@@ -1004,31 +1008,27 @@ bool handleNetPerfMeterControlMessage(MessageReader* messageReader,
          case NETPERFMETER_ADD_FLOW:
             return(handleNetPerfMeterAddFlow(
                       messageReader, controlSocket,
-                      (const NetPerfMeterAddFlowMessage*)&inputBuffer,
-                      sinfo.sinfo_assoc_id, received));
+                      (const NetPerfMeterAddFlowMessage*)&inputBuffer, received));
           break;
          case NETPERFMETER_REMOVE_FLOW:
             return(handleNetPerfMeterRemoveFlow(
                       messageReader, controlSocket,
-                      (const NetPerfMeterRemoveFlowMessage*)&inputBuffer,
-                      sinfo.sinfo_assoc_id, received));
+                      (const NetPerfMeterRemoveFlowMessage*)&inputBuffer, received));
           break;
          case NETPERFMETER_START:
             return(handleNetPerfMeterStart(
                       messageReader, controlSocket,
-                      (const NetPerfMeterStartMessage*)&inputBuffer,
-                      sinfo.sinfo_assoc_id, received));
+                      (const NetPerfMeterStartMessage*)&inputBuffer, received));
           break;
          case NETPERFMETER_STOP:
             return(handleNetPerfMeterStop(
                       messageReader, controlSocket,
-                      (const NetPerfMeterStopMessage*)&inputBuffer,
-                      sinfo.sinfo_assoc_id, received));
+                      (const NetPerfMeterStopMessage*)&inputBuffer, received));
           break;
          default:
             std::cerr << "ERROR: Received invalid control message of type "
                      << (unsigned int)header->Type << "!" << std::endl;
-            sendAbort(controlSocket, sinfo.sinfo_assoc_id);
+            sendAbort(controlSocket);
             return(false);
           break;
       }
@@ -1042,9 +1042,8 @@ void handleNetPerfMeterIdentify(const NetPerfMeterIdentifyMessage* identifyMsg,
                                 const int                          sd,
                                 const sockaddr_union*              from)
 {
-   int          controlSocketDescriptor;
-   sctp_assoc_t controlAssocID;
-   Flow*        flow;
+   int   controlSocketDescriptor;
+   Flow* flow;
 
    OutputFileFormat vectorFileFormat = OFF_Plain;
    if(identifyMsg->Header.Flags & NPMIF_COMPRESS_VECTORS) {
@@ -1059,11 +1058,10 @@ void handleNetPerfMeterIdentify(const NetPerfMeterIdentifyMessage* identifyMsg,
                                                         ntohs(identifyMsg->StreamID),
                                                         sd, from,
                                                         vectorFileFormat,
-                                                        controlSocketDescriptor,
-                                                        controlAssocID);
-   if((controlAssocID != 0) && (flow != NULL)) {
+                                                        controlSocketDescriptor);
+   if(flow != NULL) {
       const bool success = flow->configureSocket(sd);
-      sendNetPerfMeterAcknowledge(controlSocketDescriptor, controlAssocID,
+      sendNetPerfMeterAcknowledge(controlSocketDescriptor,
                                   ntoh64(identifyMsg->MeasurementID),
                                   ntohl(identifyMsg->FlowID),
                                   ntohs(identifyMsg->StreamID),
@@ -1071,7 +1069,7 @@ void handleNetPerfMeterIdentify(const NetPerfMeterIdentifyMessage* identifyMsg,
                                                       NETPERFMETER_STATUS_ERROR);
    }
    else {
-      // No known flow -> no assoc ID to send response to!
+      // No known flow -> no association to send response to!
       std::cerr << "WARNING: NETPERFMETER_IDENTIFY_FLOW message for unknown flow!" << std::endl;
    }
 }
@@ -1079,7 +1077,6 @@ void handleNetPerfMeterIdentify(const NetPerfMeterIdentifyMessage* identifyMsg,
 
 // ###### Send NETPERFMETER_ACKNOWLEDGE to remote node ######################
 bool sendNetPerfMeterAcknowledge(int            controlSocket,
-                                 sctp_assoc_t   assocID,
                                  const uint64_t measurementID,
                                  const uint32_t flowID,
                                  const uint16_t streamID,
@@ -1097,8 +1094,7 @@ bool sendNetPerfMeterAcknowledge(int            controlSocket,
 
    sctp_sndrcvinfo sinfo;
    memset(&sinfo, 0, sizeof(sinfo));
-   sinfo.sinfo_assoc_id = assocID;
-   sinfo.sinfo_ppid     = htonl(PPID_NETPERFMETER_CONTROL);
+   sinfo.sinfo_ppid = htonl(PPID_NETPERFMETER_CONTROL);
 
    return(sctp_send(controlSocket, &ackMsg, sizeof(ackMsg), &sinfo, 0) > 0);
 }
