@@ -1,7 +1,8 @@
-/* $Id: flow.cc 1561 2012-12-04 16:17:41Z adhari $
+/* $Id: flow.cc 1880 2014-07-10 12:35:32Z dreibh $
  *
  * Network Performance Meter
- * Copyright (C) 2009-2012 by Thomas Dreibholz
+ * Copyright (C) 2013 by Sebastian Wallat (TCP No delay)
+ * Copyright (C) 2009-2014 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +17,8 @@
  * You should have relReceived a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Contact: dreibh@iem.uni-due.de
+ * Contact: sebastian.wallat@uni-die.de
+ *          dreibh@iem.uni-due.de
  */
 
 #include "flow.h"
@@ -28,6 +30,7 @@
 #include <poll.h>
 #include <assert.h>
 #include <math.h>
+#include <netinet/tcp.h>
 
 
 // Flow Manager Singleton object
@@ -326,12 +329,10 @@ Flow* FlowManager::identifySocket(const uint64_t         measurementID,
                                   const int              socketDescriptor,
                                   const sockaddr_union*  from,
                                   const OutputFileFormat vectorFileFormat,
-                                  int&                   controlSocketDescriptor,
-                                  sctp_assoc_t&          controlAssocID)
+                                  int&                   controlSocketDescriptor)
 {
    bool success            = false;
    controlSocketDescriptor = -1;
-   controlAssocID          = 0;
 
    lock();
    Flow* flow = findFlow(measurementID, flowID, streamID);
@@ -342,7 +343,6 @@ Flow* FlowManager::identifySocket(const uint64_t         measurementID,
       flow->RemoteAddress        = *from;
       flow->RemoteAddressIsValid = true;
       controlSocketDescriptor    = flow->RemoteControlSocketDescriptor;
-      controlAssocID             = flow->RemoteControlAssocID;
       success = flow->initializeVectorFile(NULL, vectorFileFormat);
       flow->unlock();
       removeSocket(socketDescriptor, false);   // Socket is now managed as flow!
@@ -643,6 +643,7 @@ void FlowManager::writeVectorStatistics(const uint64_t           measurementID,
           currentTotalStats  = currentTotalStats + flow->CurrentBandwidthStats;
           CurrentGlobalStats = CurrentGlobalStats + relStats;
 
+          const unsigned long long line = vectorFile.getLine();
           vectorFile.printf(
             "%06llu %llu %1.6f %1.6f\t%u \"%s\" %1.3f\t"
                "\"Sent\"     %llu %llu %llu\t%llu %llu %llu\n"
@@ -651,27 +652,30 @@ void FlowManager::writeVectorStatistics(const uint64_t           measurementID,
             "%06llu %llu %1.6f %1.6f\t%u \"%s\" %1.3f\t"
                "\"Lost\"     %llu %llu %llu\t%llu %llu %llu\n",
 
-            vectorFile.nextLine(), now, (double)(now - firstStatisticsEvent) / 1000000.0, duration,
+            line + 1, now, (double)(now - firstStatisticsEvent) / 1000000.0, duration,
                flow->FlowID, flow->TrafficSpec.Description.c_str(), flow->Jitter,
                flow->CurrentBandwidthStats.TransmittedBytes, flow->CurrentBandwidthStats.TransmittedPackets, flow->CurrentBandwidthStats.TransmittedFrames,
                relStats.TransmittedBytes, relStats.TransmittedPackets, relStats.TransmittedFrames,
 
-            vectorFile.nextLine(), now, (double)(now - firstStatisticsEvent) / 1000000.0, duration,
+            line + 2, now, (double)(now - firstStatisticsEvent) / 1000000.0, duration,
                flow->FlowID, flow->TrafficSpec.Description.c_str(), flow->Jitter,
                flow->CurrentBandwidthStats.ReceivedBytes, flow->CurrentBandwidthStats.ReceivedPackets, flow->CurrentBandwidthStats.ReceivedFrames,
                relStats.ReceivedBytes, relStats.ReceivedPackets, relStats.ReceivedFrames,
 
-            vectorFile.nextLine(), now, (double)(now - firstStatisticsEvent) / 1000000.0, duration,
+            line + 3, now, (double)(now - firstStatisticsEvent) / 1000000.0, duration,
                flow->FlowID, flow->TrafficSpec.Description.c_str(), flow->Jitter,
                flow->CurrentBandwidthStats.LostBytes, flow->CurrentBandwidthStats.LostPackets, flow->CurrentBandwidthStats.LostFrames,
                relStats.LostBytes, relStats.LostPackets, relStats.LostFrames);
 
+          vectorFile.nextLine(); vectorFile.nextLine(); vectorFile.nextLine();
+
           flow->LastBandwidthStats = flow->CurrentBandwidthStats;
           flow->unlock();
        }
    }
 
    // ====== Write total statistics =========================================
+   const unsigned long long line = vectorFile.getLine();
    const FlowBandwidthStats relTotalStats = currentTotalStats - lastTotalStats;
       vectorFile.printf(
       "%06llu %llu %1.6f %1.6f\t-1 \"Total\" 0\t"
@@ -681,18 +685,19 @@ void FlowManager::writeVectorStatistics(const uint64_t           measurementID,
       "%06llu %llu %1.6f %1.6f\t-1 \"Total\" 0\t"
          "\"Lost\"     %llu %llu %llu\t%llu %llu %llu\n",
 
-      vectorFile.nextLine(), now, (double)(now - firstStatisticsEvent) / 1000000.0, duration,
+      line + 1, now, (double)(now - firstStatisticsEvent) / 1000000.0, duration,
          currentTotalStats.TransmittedBytes, currentTotalStats.TransmittedPackets, currentTotalStats.TransmittedFrames,
          relTotalStats.TransmittedBytes, relTotalStats.TransmittedPackets, relTotalStats.TransmittedFrames,
 
-      vectorFile.nextLine(), now, (double)(now - firstStatisticsEvent) / 1000000.0, duration,
+      line + 2, now, (double)(now - firstStatisticsEvent) / 1000000.0, duration,
          currentTotalStats.ReceivedBytes, currentTotalStats.ReceivedPackets, currentTotalStats.ReceivedFrames,
          relTotalStats.ReceivedBytes, relTotalStats.ReceivedPackets, relTotalStats.ReceivedFrames,
 
-      vectorFile.nextLine(), now, (double)(now - firstStatisticsEvent) / 1000000.0, duration,
+      line + 3, now, (double)(now - firstStatisticsEvent) / 1000000.0, duration,
          currentTotalStats.LostBytes, currentTotalStats.LostPackets, currentTotalStats.LostFrames,
          relTotalStats.LostBytes, relTotalStats.LostPackets, relTotalStats.LostFrames);
 
+    vectorFile.nextLine(); vectorFile.nextLine(); vectorFile.nextLine();
 
    // ====== Return global values (all measurements) for displaying =========
    globalStats     = CurrentGlobalStats;
@@ -833,8 +838,7 @@ Flow::Flow(const uint64_t         measurementID,
            const uint32_t         flowID,
            const uint16_t         streamID,
            const FlowTrafficSpec& trafficSpec,
-           const int              controlSocketDescriptor,
-           const sctp_assoc_t     controlAssocID)
+           const int              controlSocketDescriptor)
 {
    lock();
    MeasurementID                 = measurementID;
@@ -844,7 +848,6 @@ Flow::Flow(const uint64_t         measurementID,
    SocketDescriptor              = -1;
    OriginalSocketDescriptor      = false;
    RemoteControlSocketDescriptor = controlSocketDescriptor;
-   RemoteControlAssocID          = controlAssocID;
    RemoteAddressIsValid          = false;
 
    InputStatus                   = WaitingForStartup;
@@ -899,7 +902,7 @@ void Flow::resetStatistics()
 void Flow::print(std::ostream& os, const bool printStatistics)
 {
    lock();
-   if((OriginalSocketDescriptor) || (RemoteControlAssocID != 0)) {
+   if(OriginalSocketDescriptor) {
       if(TrafficSpec.Protocol != IPPROTO_SCTP) {
          os << "+ " << getProtocolName(TrafficSpec.Protocol) << " Flow (Flow ID #"
             << FlowID << " \"" << TrafficSpec.Description << "\"):" << std::endl;
@@ -1049,9 +1052,6 @@ void Flow::deactivate(const bool asyncStop)
          }
          else {
             const int shutdownOkay = ext_shutdown(SocketDescriptor, 2);
-/*
-            const int shutdownOkay = sctp_sendmsg(SocketDescriptor, NULL, 0, NULL, 0, 0, SCTP_EOF, 0, 0, 0);
-*/
             if(shutdownOkay < 0) {
                perror("WARNING: Cannut shut association down");
             }
@@ -1200,39 +1200,81 @@ void Flow::run()
 // ###### Configure socket parameters #######################################
 bool Flow::configureSocket(const int socketDescriptor)
 {
-    if(TrafficSpec.RcvBufferSize != 0) {
-       int bufferSize = TrafficSpec.RcvBufferSize;
-       if(ext_setsockopt(socketDescriptor, SOL_SOCKET, SO_RCVBUF, &bufferSize, sizeof(bufferSize)) < 0) {
-           std::cerr << "ERROR: Failed to configure receive buffer size on SCTP socket (SO_RCVBUF option) - "
-                     << strerror(errno) << "!" << std::endl;
-           return(false);
-       }
-    }
-    if(TrafficSpec.SndBufferSize != 0) {
-       int bufferSize = TrafficSpec.SndBufferSize;
-       if(ext_setsockopt(socketDescriptor, SOL_SOCKET, SO_SNDBUF, &bufferSize, sizeof(bufferSize)) < 0) {
-           std::cerr << "ERROR: Failed to configure send buffer size on SCTP socket (SO_SNDBUF option) - "
-                     << strerror(errno) << "!" << std::endl;
-           return(false);
-       }
-    }
-    if(TrafficSpec.Protocol == IPPROTO_SCTP) {
-#ifdef SCTP_CMT_ON_OFF
-        struct sctp_assoc_value cmtOnOff;
-        cmtOnOff.assoc_id    = 0;
-        cmtOnOff.assoc_value = TrafficSpec.CMT;
-        if(ext_setsockopt(socketDescriptor, IPPROTO_SCTP, SCTP_CMT_ON_OFF, &cmtOnOff, sizeof(cmtOnOff)) < 0) {
-          if(TrafficSpec.CMT != NPAF_PRIMARY_PATH) {
-              std::cerr << "ERROR: Failed to configure CMT usage on SCTP socket (SCTP_CMT_ON_OFF option) - "
-                        << strerror(errno) << "!" << std::endl;
-             return(false);
-          }
-        }
+   if(TrafficSpec.RcvBufferSize != 0) {
+      int bufferSize = TrafficSpec.RcvBufferSize;
+      if(ext_setsockopt(socketDescriptor, SOL_SOCKET, SO_RCVBUF, &bufferSize, sizeof(bufferSize)) < 0) {
+         std::cerr << "ERROR: Failed to configure receive buffer size on SCTP socket (SO_RCVBUF option) - "
+                  << strerror(errno) << "!" << std::endl;
+         return(false);
+      }
+   }
+   if(TrafficSpec.SndBufferSize != 0) {
+      int bufferSize = TrafficSpec.SndBufferSize;
+      if(ext_setsockopt(socketDescriptor, SOL_SOCKET, SO_SNDBUF, &bufferSize, sizeof(bufferSize)) < 0) {
+         std::cerr << "ERROR: Failed to configure send buffer size on SCTP socket (SO_SNDBUF option) - "
+                  << strerror(errno) << "!" << std::endl;
+         return(false);
+      }
+   }
+   if( (TrafficSpec.Protocol == IPPROTO_TCP) || (TrafficSpec.Protocol == IPPROTO_MPTCP) ) {
+      const int noDelayOption = (TrafficSpec.NoDelay == true) ? 1 : 0;
+      if (ext_setsockopt(socketDescriptor, IPPROTO_TCP, TCP_NODELAY, (const char*)&noDelayOption, sizeof(noDelayOption)) < 0) {
+         std::cerr << "ERROR: Failed to set TCP_NODELAY on socket - "
+               << strerror(errno) << "!" << std::endl;
+         return(false);
+      }
+
+      if(TrafficSpec.Protocol == IPPROTO_MPTCP) {
+         // FIXME! Add proper, platform-independent code here!
+#ifndef __linux__
+#warning MPTCP is currently only available on Linux!
 #else
-        if(TrafficSpec.CMT != NPAF_PRIMARY_PATH) {
-          std::cerr << "ERROR: CMT usage on SCTP socket configured, but not supported by this system!" << std::endl;
-          return(false);
-        }
+
+         const int debugOption = (TrafficSpec.Debug == true) ? 1 : 0;
+         if (ext_setsockopt(socketDescriptor, IPPROTO_TCP, TCP_MULTIPATH_DEBUG, (const char*)&debugOption, sizeof(debugOption)) < 0) {
+            std::cerr << "WARNING: Failed to set TCP_MULTIPATH_DEBUG on socket - "
+                      << strerror(errno) << "!" << std::endl;
+         }
+
+         const int nDiffPorts = (int)TrafficSpec.NDiffPorts;
+         if (ext_setsockopt(socketDescriptor, IPPROTO_TCP, TCP_MULTIPATH_NDIFFPORTS, (const char*)&nDiffPorts, sizeof(nDiffPorts)) < 0) {
+            std::cerr << "WARNING: Failed to set TCP_MULTIPATH_NDIFFPORTS on socket - "
+                      << strerror(errno) << "!" << std::endl;
+         }
+
+         const char* pathMgr = TrafficSpec.PathMgr.c_str();
+         if (ext_setsockopt(socketDescriptor, IPPROTO_TCP, TCP_MULTIPATH_PATHMANAGER, pathMgr, strlen(pathMgr)) < 0) {
+            std::cerr << "WARNING: Failed to set TCP_MULTIPATH_PATHMANAGER on socket - "
+                      << strerror(errno) << "!" << std::endl;
+         }
+#endif
+      }
+   }
+   else if(TrafficSpec.Protocol == IPPROTO_SCTP) {
+      if (TrafficSpec.NoDelay) {
+         const int noDelayOption = 1;
+         if (ext_setsockopt(socketDescriptor, IPPROTO_SCTP, SCTP_NODELAY, (const char*)&noDelayOption, sizeof(noDelayOption)) < 0) {
+            std::cerr << "ERROR: Failed to set SCTP_NODELAY on socket - "
+                  << strerror(errno) << "!" << std::endl;
+            return(false);
+         }
+      }
+#ifdef SCTP_CMT_ON_OFF
+      struct sctp_assoc_value cmtOnOff;
+      cmtOnOff.assoc_id    = 0;
+      cmtOnOff.assoc_value = TrafficSpec.CMT;
+      if(ext_setsockopt(socketDescriptor, IPPROTO_SCTP, SCTP_CMT_ON_OFF, &cmtOnOff, sizeof(cmtOnOff)) < 0) {
+         if(TrafficSpec.CMT != NPAF_PRIMARY_PATH) {
+            std::cerr << "ERROR: Failed to configure CMT usage on SCTP socket (SCTP_CMT_ON_OFF option) - "
+                     << strerror(errno) << "!" << std::endl;
+            return(false);
+         }
+      }
+#else
+      if(TrafficSpec.CMT != NPAF_PRIMARY_PATH) {
+         std::cerr << "ERROR: CMT usage on SCTP socket configured, but not supported by this system!" << std::endl;
+         return(false);
+      }
 #endif
    }
 #ifdef HAVE_DCCP
