@@ -1,7 +1,8 @@
-/* $Id: flowtrafficspec.cc 1561 2012-12-04 16:17:41Z adhari $
+/* $Id: flowtrafficspec.cc 1877 2014-07-09 13:44:24Z dreibh $
  *
  * Network Performance Meter
- * Copyright (C) 2009-2012 by Thomas Dreibholz
+ * Copyright (C) 2013 by Sebastian Wallat (TCP No delay)
+ * Copyright (C) 2009-2014 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +17,8 @@
  * You should have relReceived a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Contact: dreibh@iem.uni-due.de
+ * Contact: sebastian.wallat@uni-due.de
+ *          dreibh@iem.uni-due.de
  */
 
 #include "flowtrafficspec.h"
@@ -122,8 +124,19 @@ void FlowTrafficSpec::print(std::ostream& os) const
       os << "*0 ";
    }
    os << "}" << std::endl;
+
    os << "      - Error on Abort:      "
-      << ((ErrorOnAbort == true) ? "yes" : "no") << std::endl;
+      << ((ErrorOnAbort == true) ? "yes" : "no") << std::endl
+      << "      - Debug:               "
+      << ((Debug == true) ? "yes" : "no") << std::endl
+      << "      - No Delay:            "
+      << ((NoDelay == true) ? "yes" : "no") << std::endl;
+
+//    if(Protocol == IPPROTO_MPTCP) {
+      os << "      Number of Diff. Ports: " << NDiffPorts << std::endl
+         << "      Path Manager:          " << PathMgr    << std::endl;
+//    }
+
    os << "      - CMT:                 #" << (unsigned int)CMT << " ";
    switch(CMT) {
       case NPAF_PRIMARY_PATH:
@@ -159,6 +172,10 @@ void FlowTrafficSpec::reset()
    RetransmissionTrials     = ~0;
    RetransmissionTrialsInMS = true;
    ErrorOnAbort             = true;
+   Debug                    = false;
+   NoDelay                  = false;
+   NDiffPorts               = 4;
+   PathMgr                  = "default";
    CMT                      = 0x00;
    CCID                     = 0x00;
    for(size_t i = 0;i < NETPERFMETER_RNG_INPUT_PARAMETERS;i++) {
