@@ -1,7 +1,8 @@
-/* $Id: flowtrafficspec.h 1561 2012-12-04 16:17:41Z adhari $
+/* $Id: flowtrafficspec.h 1877 2014-07-09 13:44:24Z dreibh $
  *
  * Network Performance Meter
- * Copyright (C) 2009-2012 by Thomas Dreibholz
+ * Copyright (C) 2013 by Sebastian Wallat (TCP No delay)
+ * Copyright (C) 2009-2014 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +17,8 @@
  * You should have relReceived a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Contact: dreibh@iem.uni-due.de
+ * Contact: sebastian.wallat@uni-due.de
+ *          dreibh@iem.uni-due.de
  */
 
 #ifndef FLOWTRAFFICSPEC_H
@@ -73,6 +75,12 @@ class FlowTrafficSpec
 
    uint8_t                CMT;
    uint8_t                CCID;
+   
+   uint16_t               NDiffPorts;
+   std::string            PathMgr;
+
+   bool                   Debug;
+   bool                   NoDelay;
 
    bool                   ErrorOnAbort;
 };
