@@ -1,7 +1,7 @@
-/* $Id: messagereader.cc 1140 2011-11-23 06:43:27Z dreibh $
+/* $Id: messagereader.cc 1874 2014-07-09 11:47:52Z dreibh $
  *
  * Network Performance Meter
- * Copyright (C) 2009-2012 by Thomas Dreibholz
+ * Copyright (C) 2009-2014 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -87,6 +87,18 @@ bool MessageReader::registerSocket(const int    protocol,
 }
 
 
+// ###### Get all socket descriptors ########################################
+size_t MessageReader::getAllSDs(int* sds, const size_t maxEntries)
+{
+   assert(maxEntries >= SocketMap.size());
+   size_t count = 0;
+   for(std::map<int, Socket*>::iterator iterator = SocketMap.begin(); iterator != SocketMap.end(); iterator++) {
+      sds[count++] = iterator->second->SocketDescriptor;
+   }
+   return(count);
+}
+
+
 // ###### Deregister a socket ###############################################
 bool MessageReader::deregisterSocket(const int sd)
 {
@@ -195,7 +207,8 @@ ssize_t MessageReader::receiveMessage(const int        sd,
 #ifdef DEBUG_MESSAGEREADER
                   printf("Socket %d:   T=%u F=%02x L=%u   [Header]\n",
                           socket->SocketDescriptor,
-                          (unsigned int)header->Type, (unsigned int)header->Flags, ntohs(header->Length));
+                          (unsigned int)header->Type, (unsigned int)header->Flags,
+                         ntohs(header->Length));
 #endif
                   socket->MessageSize = ntohs(header->Length);
                   if(socket->MessageSize < sizeof(TLVHeader)) {
