@@ -1,7 +1,7 @@
-/* $Id: messagereader.h 1431 2012-07-02 11:21:07Z dreibh $
+/* $Id: messagereader.h 1874 2014-07-09 11:47:52Z dreibh $
  *
  * Network Performance Meter
- * Copyright (C) 2009-2012 by Thomas Dreibholz
+ * Copyright (C) 2009-2014 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,6 +52,11 @@ class MessageReader
                           socklen_t*       fromSize = NULL,
                           sctp_sndrcvinfo* sinfo    = NULL,
                           int*             msgFlags = NULL);
+   size_t getAllSDs(int* sds, const size_t maxEntries);
+   
+   inline size_t size() {
+      return(SocketMap.size()); 
+   }
 
    // ====== Private Data ===================================================
    private:
