@@ -1,7 +1,8 @@
-/* $Id: netperfmeter.cc 1359 2012-04-12 12:25:06Z dreibh $
+/* $Id: netperfmeter.cc 1877 2014-07-09 13:44:24Z dreibh $
  *
  * Network Performance Meter
- * Copyright (C) 2009-2012 by Thomas Dreibholz
+ * Copyright (C) 2013 by Sebastian Wallat (TCP No delay)
+ * Copyright (C) 2009-2014 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +17,8 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Contact: dreibh@iem.uni-due.de
+ * Contact: sebastian.wallat@uni-due.de
+ *          dreibh@iem.uni-due.de
  */
 
 #include <stdio.h>
@@ -34,17 +36,6 @@
 #include "control.h"
 #include "transfer.h"
 
-#ifdef HAVE_MPTCP
-// ====== Constants for draft-ietf-mptcp-api-04 =============================
-// FIXME: This should really be set within a standard include file!
-#warning linux/tcp.h does not work with C++ code. Using ugly workaround!
-// #include <linux/tcp.h>
-#define TCP_MULTIPATH_ENABLE   19
-#define TCP_MULTIPATH_ADD      20
-#define TCP_MULTIPATH_REMOVE   21
-// ==========================================================================
-#endif
-
 
 using namespace std;
 
@@ -53,16 +44,18 @@ using namespace std;
 static unsigned int   gLocalAddresses  = 0;
 static sockaddr_union gLocalAddressArray[MAX_LOCAL_ADDRESSES];
 
-static const char*    gActiveNodeName  = "Client";
-static const char*    gPassiveNodeName = "Server";
-static int            gControlSocket   = -1;
-static int            gTCPSocket       = -1;
-static int            gMPTCPSocket     = -1;
-static int            gUDPSocket       = -1;
-static int            gSCTPSocket      = -1;
-static int            gDCCPSocket      = -1;
-static double         gRuntime         = -1.0;
-static bool           gStopTimeReached = false;
+static const char*    gActiveNodeName   = "Client";
+static const char*    gPassiveNodeName  = "Server";
+static bool           gControlOverTCP   = false;
+static int            gControlSocket    = -1;
+static int            gControlSocketTCP = -1;
+static int            gTCPSocket        = -1;
+static int            gMPTCPSocket      = -1;
+static int            gUDPSocket        = -1;
+static int            gSCTPSocket       = -1;
+static int            gDCCPSocket       = -1;
+static double         gRuntime          = -1.0;
+static bool           gStopTimeReached  = false;
 MessageReader         gMessageReader;
 
 
@@ -73,6 +66,9 @@ bool handleGlobalParameter(char* parameter)
    if(strncmp(parameter, "-runtime=", 9) == 0) {
       gRuntime = atof((const char*)&parameter[9]);
    }
+   else if(strcmp(parameter, "-control-over-tcp") == 0) {
+      gControlOverTCP = true;
+   }
    else if(strncmp(parameter, "-activenodename=", 16) == 0) {
       gActiveNodeName = (const char*)&parameter[16];
    }
@@ -362,8 +358,64 @@ static const char* parseTrafficSpecOption(const char*      parameters,
       }
       n = m;
    }
+   else if(strncmp(parameters, "nodelay=", 8) == 0) {
+      if(strncmp((const char*)&parameters[8], "on", 2) == 0) {
+         trafficSpec.NoDelay = true;
+         n = 8 + 2;
+      }
+      else if(strncmp((const char*)&parameters[8], "off", 3) == 0) {
+         trafficSpec.NoDelay = false;
+         n = 8 + 3;
+      }
+      else {
+         cerr << "ERROR: Invalid \"nodelay\" setting: " << (const char*)&parameters[8] << "!" << std::endl;
+         exit(1);
+      }
+   }
+   else if(strncmp(parameters, "debug=", 6) == 0) {
+      if(strncmp((const char*)&parameters[6], "on", 2) == 0) {
+         trafficSpec.Debug = true;
+         n = 6 + 2;
+      }
+      else if(strncmp((const char*)&parameters[6], "off", 3) == 0) {
+         trafficSpec.Debug = false;
+         n = 6 + 3;
+      }
+      else {
+         cerr << "ERROR: Invalid \"debug\" setting: " << (const char*)&parameters[6] << "!" << std::endl;
+         exit(1);
+      }
+   }
+   else if(strncmp(parameters, "ndiffports=", 11) == 0) {
+      unsigned int nDiffPorts;
+      int          pos;
+      if(sscanf((const char*)&parameters[11], "%u%n", &nDiffPorts, &pos) == 1) {
+         trafficSpec.NDiffPorts = (uint16_t)nDiffPorts;
+         n = 11 + pos;
+      }
+   }
+   else if(strncmp(parameters, "pathmgr=", 8) == 0) {
+      char   pathMgr[NETPERFMETER_PATHMGR_LENGTH + 1];
+      size_t i = 0;
+      while(i < NETPERFMETER_PATHMGR_LENGTH) {
+         if( (parameters[8 + i] == ':') ||
+             (parameters[8 + i] == 0x00) ) {
+            break;
+         }
+         pathMgr[i] = parameters[8 + i];
+         i++;
+      }
+      pathMgr[i] = 0x00;
+      if( (parameters[8 + i] != ':') && (parameters[8 + i] != 0x00) ) {
+         cerr << "ERROR: Invalid \"pathmgr\" setting: " << (const char*)&parameters[8]
+              << " - name too long!" << std::endl;
+          exit(1);
+      }
+      trafficSpec.PathMgr = std::string((const char*)&pathMgr);
+      n = 8 + strlen((const char*)&pathMgr);
+   }
    else {
-      cerr << "ERROR: Invalid options " << parameters << "!" << endl;
+      cerr << "ERROR: Invalid option \"" << parameters << "\"!" << endl;
       exit(1);
    }
    if(parameters[n] == 0x00) {
@@ -425,8 +477,7 @@ static Flow* createFlow(Flow*                  previousFlow,
             if(parameters) {
                parameters = parseNextEntry(parameters, (double*)&trafficSpec.InboundFrameSize, &trafficSpec.InboundFrameSizeRng);
                if(parameters) {
-                  while( (parameters = parseTrafficSpecOption(parameters, trafficSpec, flowID)) ) {
-                  }
+                  while( (parameters = parseTrafficSpecOption(parameters, trafficSpec, flowID)) ) { }
                }
             }
          }
@@ -584,25 +635,30 @@ bool mainLoop(const bool               isActiveMode,
               const unsigned long long stopAt,
               const uint64_t           measurementID)
 {
-   pollfd                 fds[5];
-   int                    n         = 0;
-   int                    controlID = -1;
-   int                    tcpID     = -1;
-   int                    mptcpID   = -1;
-   int                    udpID     = -1;
-   int                    sctpID    = -1;
-   int                    dccpID    = -1;
-   unsigned long long     now       = getMicroTime();
+   pollfd                 fds[5 + gMessageReader.size()];
+   int                    n       = 0;
+   int                    tcpID   = -1;
+   int                    mptcpID = -1;
+   int                    udpID   = -1;
+   int                    sctpID  = -1;
+   int                    dccpID  = -1;
+   unsigned long long     now     = getMicroTime();
    std::map<int, pollfd*> pollFDIndex;
 
 
    // ====== Get parameters for poll() ======================================
-   addToPollFDs((pollfd*)&fds, gControlSocket, n, &controlID);
    addToPollFDs((pollfd*)&fds, gTCPSocket,     n, &tcpID);
    addToPollFDs((pollfd*)&fds, gMPTCPSocket,   n, &mptcpID);
    addToPollFDs((pollfd*)&fds, gUDPSocket,     n, &udpID);
    addToPollFDs((pollfd*)&fds, gSCTPSocket,    n, &sctpID);
    addToPollFDs((pollfd*)&fds, gDCCPSocket,    n, &dccpID);
+   int    controlFDSet[gMessageReader.size()];
+   size_t controlFDs = gMessageReader.getAllSDs((int*)&controlFDSet, sizeof(controlFDSet) / sizeof(int));
+   const int controlIDMin = n;
+   for(size_t i = 0; i < controlFDs; i++) {
+      addToPollFDs((pollfd*)&fds, controlFDSet[i], n, NULL);
+   }
+   const int controlIDMax = n - 1;
 
 
    // ====== Use poll() to wait for events ==================================
@@ -620,11 +676,34 @@ bool mainLoop(const bool               isActiveMode,
    if(result > 0) {
 
       // ====== Incoming control message ====================================
-      if( (controlID >= 0) && (fds[controlID].revents & POLLIN) ) {
-         const bool controlOkay = handleNetPerfMeterControlMessage(
-                                     &gMessageReader, gControlSocket);
-         if((!controlOkay) && (isActiveMode)) {
-            return(false);
+      int controlID;
+      for(controlID = controlIDMin; controlID <= controlIDMax; controlID++) {
+         if(fds[controlID].revents & POLLIN) {
+            if( (isActiveMode == false) &&
+                (fds[controlID].fd == gControlSocket) ) {
+               const int newSD = ext_accept(gControlSocket, NULL, 0);
+               if(newSD >= 0) {
+                  gMessageReader.registerSocket(IPPROTO_SCTP, newSD);
+               }
+            }
+            else if( (isActiveMode == false) &&
+                (fds[controlID].fd == gControlSocketTCP) ) {
+               const int newSD = ext_accept(gControlSocketTCP, NULL, 0);
+               if(newSD >= 0) {
+                  gMessageReader.registerSocket(IPPROTO_TCP, newSD);
+               }
+            }
+            else {
+               const bool controlOkay = handleNetPerfMeterControlMessage(
+                                           &gMessageReader, fds[controlID].fd);
+               if(!controlOkay) {
+                  if(isActiveMode) {
+                     return(false);
+                  }
+                  gMessageReader.deregisterSocket(fds[controlID].fd);
+                  ext_close(fds[controlID].fd);
+               }
+            }
          }
       }
 
@@ -684,29 +763,39 @@ void passiveMode(int argc, char** argv, const uint16_t localPort)
    }
    printGlobalParameters();
 
-
    // ====== Initialize control socket ======================================
-   gControlSocket = createAndBindSocket(AF_UNSPEC, SOCK_SEQPACKET, IPPROTO_SCTP, localPort + 1,
-                                        0, NULL, true);
+   gControlSocket = createAndBindSocket(AF_UNSPEC, SOCK_STREAM, IPPROTO_SCTP,
+                                        localPort + 1, 0, NULL, true);
    if(gControlSocket < 0) {
       cerr << "ERROR: Failed to create and bind SCTP socket for control port - "
            << strerror(errno) << "!" << endl;
       exit(1);
    }
    sctp_event_subscribe events;
-   memset((char*)&events, 0 ,sizeof(events));
-   events.sctp_data_io_event     = 1;
-   events.sctp_association_event = 1;
-   if(ext_setsockopt(gControlSocket, IPPROTO_SCTP, SCTP_EVENTS, &events, sizeof(events)) < 0) {
-      cerr << "ERROR: Failed to configure events on control socket - "
-           << strerror(errno) << "!" << endl;
-      exit(1);
+   if(!gControlOverTCP) {
+      memset((char*)&events, 0 ,sizeof(events));
+      events.sctp_data_io_event     = 1;
+      events.sctp_association_event = 1;
+      if(ext_setsockopt(gControlSocket, IPPROTO_SCTP, SCTP_EVENTS, &events, sizeof(events)) < 0) {
+         cerr << "ERROR: Failed to configure events on SCTP control socket - "
+              << strerror(errno) << "!" << endl;
+         exit(1);
+      }
    }
    gMessageReader.registerSocket(IPPROTO_SCTP, gControlSocket);
 
+   gControlSocketTCP = createAndBindSocket(AF_UNSPEC, SOCK_STREAM, IPPROTO_TCP,
+                                           localPort + 1, 0, NULL, true);
+   if(gControlSocketTCP < 0) {
+      cerr << "ERROR: Failed to create and bind TCP socket for control port - "
+           << strerror(errno) << "!" << endl;
+      exit(1);
+   }
+   gMessageReader.registerSocket(IPPROTO_TCP, gControlSocketTCP);
+
    // ====== Initialize data socket for each protocol =======================
    gTCPSocket = createAndBindSocket(AF_UNSPEC, SOCK_STREAM, IPPROTO_TCP, localPort,
-                                    gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, true);
+                                    gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, true);                                    
    if(gTCPSocket < 0) {
       cerr << "ERROR: Failed to create and bind TCP socket - "
            << strerror(errno) << "!" << endl;
@@ -721,10 +810,15 @@ void passiveMode(int argc, char** argv, const uint16_t localPort)
            << strerror(errno) << "!" << endl;
       exit(1);
    }
+
+// FIXME! Add proper, platform-independent code here!
+#ifndef __linux__
+#warning MPTCP is currently only available on Linux!
+#else
    int cmtOnOff = 1;
    if(ext_setsockopt(gMPTCPSocket, IPPROTO_TCP, TCP_MULTIPATH_ENABLE, &cmtOnOff, sizeof(cmtOnOff)) < 0) {
-      std::cerr << "NOTE: Compiled with MPTCP support, but unable to initialize it: "
-                << strerror(errno) << "!" << endl;
+      cerr << "NOTE: Compiled with MPTCP support, but unable to initialize it: "
+           << strerror(errno) << "!" << endl;
       ext_close(gMPTCPSocket);
       gMPTCPSocket = -1;
    }
@@ -736,6 +830,7 @@ void passiveMode(int argc, char** argv, const uint16_t localPort)
          exit(1);
       }
    }
+#endif
 #endif
 
    gUDPSocket = createAndBindSocket(AF_UNSPEC, SOCK_DGRAM, IPPROTO_UDP, localPort,
@@ -810,6 +905,8 @@ void passiveMode(int argc, char** argv, const uint16_t localPort)
 
 
    // ====== Clean up =======================================================
+   gMessageReader.deregisterSocket(gControlSocketTCP);
+   ext_close(gControlSocketTCP);
    gMessageReader.deregisterSocket(gControlSocket);
    ext_close(gControlSocket);
    ext_close(gTCPSocket);
@@ -859,9 +956,15 @@ void activeMode(int argc, char** argv)
    }
 
    // ====== Initialize control socket ======================================
-   gControlSocket = ext_socket(controlAddress.sa.sa_family, SOCK_STREAM, IPPROTO_SCTP);
+   for(int i = 2;i < argc;i++) {
+      if(strcmp(argv[i], "-control-over-tcp") == 0) {
+         gControlOverTCP = true;
+      }
+   }
+   const int controlSocketProtocol = (gControlOverTCP == false) ? IPPROTO_SCTP : IPPROTO_TCP;
+   gControlSocket = ext_socket(controlAddress.sa.sa_family, SOCK_STREAM, controlSocketProtocol);
    if(gControlSocket < 0) {
-      cerr << "ERROR: Failed to create SCTP socket for control port - "
+      cerr << "ERROR: Failed to create socket for control port - "
            << strerror(errno) << "!" << endl;
       exit(1);
    }
@@ -873,7 +976,7 @@ void activeMode(int argc, char** argv)
    if(gOutputVerbosity >= NPFOV_STATUS) {
       cout << "okay; sd=" << gControlSocket << endl << endl;
    }
-   gMessageReader.registerSocket(IPPROTO_SCTP, gControlSocket);
+   gMessageReader.registerSocket(controlSocketProtocol, gControlSocket);
 
 
    // ====== Handle command-line parameters =================================
@@ -1030,7 +1133,7 @@ int main(int argc, char** argv)
 {
    if(argc < 2) {
       cerr << "Usage: " << argv[0]
-           << " [Local Port|Remote Endpoint] {-tcp|-udp|-sctp|-dccp} {flow spec} ..."
+           << " [Local Port|Remote Endpoint] {-control-over-tcp} {-tcp|-udp|-sctp|-dccp} {flow spec} ..."
            << endl;
       exit(1);
    }
@@ -1049,8 +1152,8 @@ int main(int argc, char** argv)
       }
    }
    if(gOutputVerbosity >= NPFOV_STATUS) {
-      cout << "Network Performance Meter - Version 1.0" << endl
-           << "---------------------------------------" << endl << endl;
+      cout << "Network Performance Meter" << endl
+           << "-------------------------" << endl << endl;
    }
 
    const uint16_t localPort = atol(argv[1]);
