@@ -1,7 +1,7 @@
-/* $Id: tools.cc 1557 2012-11-29 18:57:57Z dreibh $
+/* $Id: tools.cc 1880 2014-07-10 12:35:32Z dreibh $
  *
  * Network Performance Meter
- * Copyright (C) 2009-2012 by Thomas Dreibholz
+ * Copyright (C) 2009-2014 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,8 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Contact: dreibh@iem.uni-due.de
+ * Contact: sebastian.wallat@uni-due.de
+ *          dreibh@iem.uni-due.de
  */
 
 #include "tools.h"
@@ -33,6 +34,7 @@
 #include <ctype.h>
 #include <sys/time.h>
 #include <sys/types.h>
+#include <netinet/tcp.h>
 #include <sys/socket.h>
 #include <sys/ioctl.h>
 #include <sys/uio.h>
@@ -595,9 +597,11 @@ int createAndBindSocket(const int             family,
    int socketProtocol = protocol;
 #ifdef HAVE_MPTCP
    if(socketProtocol == IPPROTO_MPTCP) {
-      socketProtocol    = IPPROTO_TCP;
-#warning Currently, MPTCP does not support TCP_MULTIPATH_ADD. Binding to ANY address instead ...
-      localAddressCount = 0;
+      socketProtocol = IPPROTO_TCP;
+      if(localAddresses > 1) {
+         printf("WARNING: Currently, MPTCP does not support TCP_MULTIPATH_ADD. Binding to ANY address instead ...\n");
+         localAddressCount = 0;
+      }
    }
 #endif
 
@@ -613,9 +617,8 @@ int createAndBindSocket(const int             family,
       setsockopt(sd, IPPROTO_IPV6, IPV6_BINDV6ONLY, (char *)&on, sizeof(on));
    }
 #endif
-
    // ====== Bind socket ====================================================
-   if( (socketProtocol != IPPROTO_SCTP) || (localAddressCount == 0) ) {
+   if(localAddressCount == 0) {
       if(ext_bind(sd, &anyAddress.sa, getSocklen(&anyAddress.sa)) != 0) {
          ext_close(sd);
          return(-3);
